syntax = "proto3";

package tests.harness.cases;
option go_package = "github.com/envoyproxy/protoc-gen-validate/tests/harness/cases/go;cases";
import "validate/validate.proto";

message BoolNone       { bool val = 1; }
message BoolConstTrue  { bool val = 1 [(validate.rules).bool.const = true]; }
message BoolConstFalse { bool val = 1 [(validate.rules).bool.const = false]; }
