syntax = "proto3";

package tests.harness;
option go_package = "github.com/envoyproxy/protoc-gen-validate/tests/harness/go;harness";

import "google/protobuf/any.proto";

message TestCase {
    google.protobuf.Any message = 1;
}

message TestResult {
    bool   Valid  = 1;
    bool   Error  = 2;
    repeated string Reasons = 3;
    bool   AllowFailure = 4;
    bool   CheckMultipleErrors = 5;
}
